﻿using System;
using System.Linq;
using HeuristicLab.BioBoost.Data;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System.Collections.Generic;
using HeuristicLab.BioBoost.Representation;

namespace HeuristicLab.BioBoost.Operators.Transformation {

  [StorableClass]
  public class FeedstockInflater : FeedstockTransformer {

    #region Construction & Cloning
    [StorableConstructor]
    protected FeedstockInflater(bool isDeserializing) : base(isDeserializing) {}
    protected FeedstockInflater(FeedstockInflater orig, Cloner cloner) : base(orig, cloner) {}
    public FeedstockInflater() {}
    public override IDeepCloneable  Clone(Cloner cloner) {
      return new FeedstockInflater(this, cloner);
    }
    #endregion

    public override IOperation Apply() {
      var product = ProductName;
      var potentials = GetFromProblemData<DoubleArray>(LayerDescriptor.PotentialsFromProblemData.NameWithPrefix(product));
      var utilizations = GetFromScope<RealVector>(LayerDescriptor.Utilizations.NameWithPrefix(product));
      if (potentials != null && utilizations != null) {
        var newUtilizations = new List<double>();
        var skipped = 0;
        for (int i = 0; i < potentials.Length; i++) {
          if (potentials[i] > 0) {
            newUtilizations.Add(utilizations[i - skipped]);
          } else {
            skipped++;
            newUtilizations.Add(0);
          }
        }
        TransformValue(LayerDescriptor.Utilizations.NameWithPrefix(product), new RealVector(newUtilizations.ToArray()));
      }
      return base.Apply();
    }
  }
}
