﻿using GeoAPI.Geometries;
using HeuristicLab.Persistence.Core;
using HeuristicLab.Persistence.Interfaces;
using NetTopologySuite.IO;
using System;
using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.BioBoost.Persistence {
  
  public class GeometrySerializer : ICompositeSerializer {

    #region ICompositeSerializer Members

    public int Priority {
      get { return 300; }
    }

    public bool CanSerialize(Type type) {
      return type.GetInterfaces().Contains(typeof (IGeometry));
    }

    public string JustifyRejection(Type type) {
      return "type does not implement interface IGeometry";
    }

    public object CreateInstance(Type type, IEnumerable<Tag> metaInfo) {
      return new WKBReader().Read((byte[]) metaInfo.First().Value);
    }

    public IEnumerable<Tag> CreateMetaInfo(object obj) {
      var g = obj as IGeometry;
      return new[] {new Tag("WKB", new WKBWriter().Write(g))};
    }

    public IEnumerable<Tag> Decompose(object obj) {
      // object was fully decomposed in CreateMetaInfo
      return new Tag[0];
    }


    public void Populate(object instance, IEnumerable<Tag> tags, Type type) {
      // object was fully populated in CreateInstance
    }


    #endregion
  }
}
