﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.IntegerVectorEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System.Linq;

namespace HeuristicLab.BioBoost.SolutionCreation {

  [StorableClass]
  public class LimitedValuesIntegerVectorCreator : IntegerVectorCreator {

    #region parameters
    public ValueLookupParameter<IntValue> NrOfValuesParameter {
      get { return (ValueLookupParameter<IntValue>) Parameters["NrOfValues"]; }
    }
    #endregion

    #region parameter values
    public IntValue NrOfValues {
      get { return NrOfValuesParameter.ActualValue; }
      set { NrOfValuesParameter.ActualValue = value; }
    }
    #endregion

    #region Construction & Cloning
    [StorableConstructor]
    protected LimitedValuesIntegerVectorCreator(bool isDeserializing) : base(isDeserializing) {}
    protected LimitedValuesIntegerVectorCreator(LimitedValuesIntegerVectorCreator orig, Cloner cloner) : base(orig, cloner) {}
    public LimitedValuesIntegerVectorCreator() {
      Parameters.Add(new ValueLookupParameter<IntValue>("NrOfValues", "Nr of distinct values to chose from.", new IntValue(1)));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new LimitedValuesIntegerVectorCreator(this, cloner);
    }
    #endregion

    protected override IntegerVector Create(IRandom random, IntValue length, IntMatrix bounds) {
      var nrOfValues = NrOfValues.Value;
      var len = length.Value;
      var values = Enumerable.Range(0, nrOfValues).Select(i => random.Next(len)).ToList();
      return new IntegerVector(Enumerable.Range(0, len).Select(i => values[random.Next(nrOfValues)]).ToArray());
    }

  }
}
