﻿using System.Collections.Generic;
using System.Diagnostics;
using HeuristicLab.Core;
using HeuristicLab.Data;

namespace HeuristicLab.BioBoost.Utils {

  public static class ValueParameterCollectionExtensions {
    /// <summary>
    /// total conversion cost including costs for required supplies minus revenue for side products.
    /// </summary>
    /// <param name="productsAndAmounts">the list of products and amounts</param>
    /// <param name="productPrices">a dictionary containing the prices to use</param>
    /// <param name="mainProductName">the main product i.e. no side product and hence excluded from this calculation</param>
    /// <returns></returns>
    public static double TotalCost(this ValueParameterCollection productsAndAmounts, IDictionary<string, double> productPrices, string mainProductName) {
      double cost = 0;
      foreach (var p in productsAndAmounts) {
        var productName = p.Name;
        var amount = p.Value as DoubleValue;
        double price;
        if (amount != null && productName != mainProductName && productPrices.TryGetValue(p.Name, out price)) {
          cost -= amount.Value*price; // as amount*price is sales revenue -amount*price is cost ;-)
        }
      }
      return cost;
    } 
  }
}
